from __future__ import annotations

from importlib.resources import files

dash_directory = files('unraphael.dash')
data_directory = files('unraphael.data')
image_directory = data_directory / 'images'
